/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.adapter.tools;

import java.awt.Rectangle;

import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpToolNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectAdapter;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectEvent;
import ca.uvic.csr.shrimp.gui.HierarchicalView.HierarchicalView;
import ca.uvic.csr.shrimp.gui.HierarchicalView.HvSvBridge;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.resource.ResourceHandler;

/**
 * Opens the HierarchicalView tool.
 * At the moment this opens the HierarchicalView in a new dialog for each open project.
 * 
 * @author Nasir Rather, Chris Callendar
 */
public class OpenHierarchicalViewAdapter extends AbstractOpenApplicationToolAdapter {

	private static final Rectangle BOUNDS = new Rectangle(DOCK_LEFT_INSIDE, DOCK_TOP_INSIDE, 400, 400);
	
	private final ShrimpProjectAdapter listener = new ShrimpProjectAdapter() {
		public void projectClosing(ShrimpProjectEvent event) {
			removeListeners(event.getProject());
		}
	};
	
	public OpenHierarchicalViewAdapter() {
		super(ShrimpProject.HIERARCHICAL_VIEW, ResourceHandler.getIcon("icon_hierarchical_view.gif"), BOUNDS);
	}
	
	protected void createTool() {
		ShrimpProject project = getProject();
		tool = new HierarchicalView(project);
		addListeners(project);
	}
	
	private void removeListeners(ShrimpProject project) {
		if (project != null) {
			try {
				project.removeProjectListener(listener);
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
				HvSvBridge.separateShrimpViewHierarchicalView(shrimpView, (HierarchicalView) tool);
			} catch (ShrimpToolNotFoundException e) {
				e.printStackTrace();
			}
		}
	}
	
	private void addListeners(ShrimpProject project) {
		if ((tool != null) && (project != null)) {
			try {
				project.addProjectListener(listener);
				final ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
				HvSvBridge.joinShrimpViewHierarchicalView(shrimpView, (HierarchicalView)tool, project);
			} catch (ShrimpToolNotFoundException e) {
				e.printStackTrace();
			}
		}
	}
	
	public void setProject(ShrimpProject project) {
		if (project != null) {
			// remove listeners from previous project
			removeListeners(getProject());
		}
		super.setProject(project);
		addListeners(project);
	}
	
	protected void afterActionHasRun() {
		if ((tool != null) && !((HierarchicalView)tool).isDisplayBeanPopulated() && isContainerVisible()) {
			((HierarchicalView)tool).populateDisplayBean();
		}
	}

	protected void updateContainerContents() {
   		super.updateContainerContents();
		if ((tool != null) && !((HierarchicalView)tool).isDisplayBeanPopulated() && isContainerVisible()) {
			((HierarchicalView)tool).populateDisplayBean();
		}
	}

}
